package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Invite;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.emoji.Emoji;
import net.dv8tion.jda.api.events.emoji.EmojiAddedEvent;
import net.dv8tion.jda.api.events.emoji.EmojiRemovedEvent;
import net.dv8tion.jda.api.events.emoji.update.EmojiUpdateNameEvent;
import net.dv8tion.jda.api.events.guild.invite.GuildInviteCreateEvent;
import net.dv8tion.jda.api.events.role.RoleCreateEvent;
import net.dv8tion.jda.api.events.role.RoleDeleteEvent;
import net.dv8tion.jda.api.events.role.update.RoleUpdateNameEvent;
import net.dv8tion.jda.api.events.role.update.RoleUpdatePermissionsEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.util.EnumSet;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "Logging - Guild Listener")
public class GuildListener implements EventListener {
    private final LogFeature logFeature;

    @Autowired
    public GuildListener(@NonNull ApplicationContext context) {
        this.logFeature = context.getBean(LogFeature.class);
    }

    @Override
    public void onGuildInviteCreate(@NonNull BatGuild guild, @NonNull Invite invite, @NonNull GuildInviteCreateEvent event) {
        log.info("Invite created in guild \"{}\" with code \"{}\"", guild.getGuild().getName(), invite.getCode());

        guild.getDiscordGuild().retrieveInvites().queue(invites -> {
            Invite newInvite = invites.stream().filter(i -> i.getCode().equals(invite.getCode())).findFirst().orElse(null);
            if (newInvite == null) {
                return;
            }
            newInvite.expand().queue(expanded -> logFeature.sendLog(guild, LogType.INVITE_CREATE, EmbedUtils.successEmbed()
                    .setDescription(new EmbedDescriptionBuilder("Invite Created")
                            .appendLine("Invite: %s".formatted(expanded.getUrl()), true)
                            .appendLine("Channel: %s".formatted(expanded.getChannel()), true)
                            .appendLine("Max Uses: `%s`".formatted(expanded.getMaxUses() == 0 ? "Infinite" : expanded.getMaxUses()), true)
                            .appendLine("Creator: %s".formatted(expanded.getInviter() == null ? "Unknown" : expanded.getInviter().getAsMention()), true)
                            .build())
                    .build()));
        });
    }

    @Override
    public void onEmojiAdd(@NonNull BatGuild guild, @NonNull Emoji emoji, @NonNull EmojiAddedEvent event) {
        log.info("Emoji \"{}\" was added in guild \"{}\"", emoji.getName(), guild.getName());
        logFeature.sendLog(guild, LogType.EMOJI_ADD, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("Emoji Added")
                        .appendLine("Emoji: %s".formatted(emoji.getFormatted()), true)
                        .appendLine("Name: `%s`".formatted(emoji.getName()), true)
                        .build())
                .build());
    }

    @Override
    public void onEmojiRemove(@NonNull BatGuild guild, @NonNull Emoji emoji, @NonNull EmojiRemovedEvent event) {
        log.info("Emoji \"{}\" was removed in guild \"{}\"", emoji.getName(), guild.getName());
        logFeature.sendLog(guild, LogType.EMOJI_REMOVE, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("Emoji Removed")
                        .appendLine("Emoji: %s".formatted(emoji.getFormatted()), true)
                        .appendLine("Name: `%s`".formatted(emoji.getName()), true)
                        .build())
                .build());
    }

    @Override
    public void onEmojiRename(@NonNull BatGuild guild, @NonNull Emoji emoji, @NonNull String oldName, @NonNull String newName, @NonNull EmojiUpdateNameEvent event) {
        log.info("Emoji \"{}\" was renamed to \"{}\" in guild \"{}\"", oldName, newName, guild.getName());
        logFeature.sendLog(guild, LogType.EMOJI_NAME_UPDATED, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("Emoji Renamed")
                        .appendLine("Emoji: %s".formatted(emoji.getFormatted()), true)
                        .appendLine("Name: `%s` -> `%s`".formatted(oldName, newName), true)
                        .build())
                .build());
    }

    @Override
    public void onRoleUpdatePermissions(@NonNull BatGuild guild, @NonNull RoleUpdatePermissionsEvent event) {
        log.info("Role \"{}\" permissions were updated in guild \"{}\"", event.getRole().getName(), guild.getName());
        EnumSet<Permission> oldPermissions = event.getOldValue();
        EnumSet<Permission> newPermissions = event.getNewValue();

        StringBuilder allowedPermissions = new StringBuilder();
        for (Permission permission : newPermissions) {
            if (!oldPermissions.contains(permission)) {
                allowedPermissions.append("`").append(permission.getName()).append("`, ");
            }
        }
        StringBuilder deniedPermissions = new StringBuilder();
        for (Permission permission : oldPermissions) {
            if (!newPermissions.contains(permission)) {
                deniedPermissions.append("`").append(permission.getName()).append("`, ");
            }
        }

        EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("Role Permissions Updated")
                .appendLine("Role: %s".formatted(event.getRole().getAsMention()), true);
        if (!allowedPermissions.isEmpty()) {
            description.appendLine("Allowed Permissions: %s".formatted(allowedPermissions.substring(0, allowedPermissions.length() - 2)), true);
        }
        if (!deniedPermissions.isEmpty()) {
            description.appendLine("Denied Permissions: %s".formatted(deniedPermissions.substring(0, deniedPermissions.length() - 2)), true);
        }

        logFeature.sendLog(guild, LogType.ROLE_PERMISSIONS_UPDATED, EmbedUtils.successEmbed()
                .setDescription(description.build())
                .build());
    }

    @Override
    public void onRoleCreate(@NonNull BatGuild guild, @NonNull RoleCreateEvent event) {
        Role role = event.getRole();
        log.info("Role \"{}\" was created in guild \"{}\"", role.getName(), guild.getName());

        logFeature.sendLog(guild, LogType.ROLE_CREATE, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("Role Created")
                        .appendLine("Role: %s".formatted(role.getAsMention()), true)
                        .appendLine("Color: %s".formatted(role.getColor() == null ? "Default" : role.getColor()), true)
                        .appendLine("Position: `%s`".formatted(role.getPosition()), true)
                        .build())
                .build());
    }

    @Override
    public void onRoleDelete(@NonNull BatGuild guild, @NonNull RoleDeleteEvent event) {
        log.info("Role \"{}\" was deleted in guild \"{}\"", event.getRole().getName(), guild.getName());
        logFeature.sendLog(guild, LogType.ROLE_DELETE, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("Role Deleted")
                        .appendLine("Role: %s".formatted(event.getRole().getAsMention()), true)
                        .appendLine("Color: %s".formatted(event.getRole().getColor() == null ? "Default" : event.getRole().getColor()), true)
                        .appendLine("Position: `%s`".formatted(event.getRole().getPosition()), true)
                        .build())
                .build());
    }

    @Override
    public void onRoleUpdateName(@NonNull BatGuild guild, @NonNull RoleUpdateNameEvent event) {
        String oldName = event.getOldName();
        String newName = event.getNewName();
        log.info("Role \"{}\" was renamed to \"{}\" in guild \"{}\"", oldName, newName, guild.getName());
        logFeature.sendLog(guild, LogType.ROLE_CONFIGURATION, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("Role Renamed")
                        .appendLine("Role: %s".formatted(event.getRole().getAsMention()), true)
                        .appendLine("Name: `%s` -> `%s`".formatted(oldName, newName), true)
                        .build())
                .build());
    }
}
